import { Collection } from "../../util/collection.js";
import { hasProperty } from "../../util/utils.js";
/**
 * Gets multiple messages from a channel.
 *
 * @param bot - The bot instance to use to make the request.
 * @param channelId - The ID of the channel from which to get the messages.
 * @param options - The parameters for the fetching of the messages.
 * @returns A collection of {@link Message} objects assorted by message ID.
 *
 * @remarks
 * Requires that the bot user be able to see the contents of the channel in which the messages were posted.
 *
 * If getting a messages from a guild channel:
 * - Requires the `READ_MESSAGE_HISTORY` permission.
 *
 * @see {@link https://discord.com/developers/docs/resources/channel#get-channel-messages}
 */
export async function getMessages(bot, channelId, options) {
    if (options?.limit && (options.limit < 0 || options.limit > 100)) {
        throw new Error(bot.constants.Errors.INVALID_GET_MESSAGES_LIMIT);
    }
    const results = await bot.rest.runMethod(bot.rest, "GET", bot.constants.routes.CHANNEL_MESSAGES(channelId, options));
    return new Collection(results.map((result) => {
        const message = bot.transformers.message(bot, result);
        return [message.id, message];
    }));
}
export function isGetMessagesAfter(options) {
    return hasProperty(options, "after");
}
export function isGetMessagesBefore(options) {
    return hasProperty(options, "before");
}
export function isGetMessagesAround(options) {
    return hasProperty(options, "around");
}
export function isGetMessagesLimit(options) {
    return hasProperty(options, "limit");
}
