import type { Bot } from "../../bot.js";
import { WithReason } from "../../mod.js";
import { Channel } from "../../transformers/channel.js";
import { OverwriteReadable } from "../../types/discordeno.js";
import { BigString, ChannelTypes, VideoQualityModes } from "../../types/shared.js";
/**
 * Edits a channel's settings.
 *
 * @param bot - The bot instance to use to make the request.
 * @param channelId - The ID of the channel to edit.
 * @param options - The parameters for the edit of the channel.
 * @returns An instance of the edited {@link Channel}.
 *
 * @remarks
 * If editing a channel of type {@link ChannelTypes.GroupDm}:
 * - Fires a _Channel Update_ gateway event.
 *
 * If editing a thread channel:
 * - Requires the `MANAGE_THREADS` permission __unless__ if setting the `archived` property to `false` when the `locked` property is also `false`, in which case only the `SEND_MESSAGES` permission is required.
 *
 * - Fires a _Thread Update_ gateway event.
 *
 * If editing a guild channel:
 * - Requires the `MANAGE_CHANNELS` permission.
 *
 * - If modifying permission overrides:
 *   - Requires the `MANAGE_ROLES` permission.
 *
 *   - Only permissions the bot user has in the guild or parent channel can be allowed/denied __unless__ the bot user has a `MANAGE_ROLES` permission override in the channel.
 *
 * - If modifying a channel of type {@link ChannelTypes.GuildCategory}:
 *     - Fires a _Channel Update_ gateway event for each child channel impacted in this change.
 * - Otherwise:
 *     - Fires a _Channel Update_ gateway event.
 */
export declare function editChannel(bot: Bot, channelId: BigString, options: ModifyChannel): Promise<Channel>;
export interface ModifyChannel extends WithReason {
    /** 1-100 character channel name */
    name?: string;
    /** The type of channel; only conversion between text and news is supported and only in guilds with the "NEWS" feature */
    type?: ChannelTypes;
    /** The position of the channel in the left-hand listing */
    position?: number | null;
    /** 0-1024 character channel topic */
    topic?: string | null;
    /** Whether the channel is nsfw */
    nsfw?: boolean | null;
    /** Amount of seconds a user has to wait before sending another message (0-21600); bots, as well as users with the permission `manage_messages` or `manage_channel`, are unaffected */
    rateLimitPerUser?: number | null;
    /** The bitrate (in bits) of the voice channel; 8000 to 96000 (128000 for VIP servers) */
    bitrate?: number | null;
    /** The user limit of the voice channel; 0 refers to no limit, 1 to 99 refers to a user limit */
    userLimit?: number | null;
    /** Channel or category-specific permissions */
    permissionOverwrites?: OverwriteReadable[] | null;
    /** Id of the new parent category for a channel */
    parentId?: BigString | null;
    /** Voice region id for the voice channel, automatic when set to null */
    rtcRegion?: string | null;
    /** The camera video quality mode of the voice channel */
    videoQualityMode?: VideoQualityModes;
    /** Whether the thread is archived */
    archived?: boolean;
    /** Duration in minutes to automatically archive the thread after recent activity */
    autoArchiveDuration?: 60 | 1440 | 4320 | 10080;
    /** When a thread is locked, only users with `MANAGE_THREADS` can unarchive it */
    locked?: boolean;
    /** whether non-moderators can add other non-moderators to a thread; only available on private threads */
    invitable?: boolean;
    /** The set of tags that can be used in a GUILD_FORUM channel */
    availableTags?: {
        /** The id of the tag */
        id: string;
        /** The name of the tag (0-20 characters) */
        name: string;
        /** Whether this tag can only be added to or removed from threads by a member with the MANAGE_THREADS permission */
        moderated: boolean;
        /** The id of a guild's custom emoji At most one of emoji_id and emoji_name may be set. */
        emojiId: string;
        /** The unicode character of the emoji */
        emojiName: string;
    }[];
    /** the emoji to show in the add reaction button on a thread in a GUILD_FORUM channel */
    defaultReactionEmoji?: {
        /** The id of a guild's custom emoji */
        emojiId: string;
        /** The unicode character of the emoji */
        emojiName: string | null;
    };
    /** the initial rate_limit_per_user to set on newly created threads in a channel. this field is copied to the thread at creation time and does not live update. */
    defaultThreadRateLimitPerUser?: number;
}
